"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assets_loader_helper_1 = require("./assets-loader.helper");
var app_helper_1 = require("./app.helper");
var dom_utils_1 = require("./dom.utils");
function loadScriptPromise(src, iframeEl) {
    return new Promise(function (resolve, reject) {
        var script = assets_loader_helper_1.default.createScriptTag(src);
        script.onload = function () {
            resolve();
        };
        script.onerror = function (err) {
            reject(err);
        };
        if (iframeEl) {
            if (iframeEl && iframeEl.contentWindow) {
                iframeEl.contentWindow.document.head.appendChild(script);
            }
        }
        else {
            document.head.appendChild(script);
        }
    });
}
var loadScriptTag = function (src, iframeEl) {
    return function () {
        return loadScriptPromise(src, iframeEl);
    };
};
var loadLinkTag = function (url, iframeEl) {
    return function () {
        return new Promise(function (resolve, reject) {
            var link = assets_loader_helper_1.default.createLinkTag(url);
            link.onload = function () {
                resolve();
            };
            link.onerror = function (err) {
                reject(err);
            };
            if (iframeEl) {
                if (iframeEl && iframeEl.contentWindow) {
                    iframeEl.contentWindow.document.head.appendChild(link);
                }
            }
            else {
                document.head.appendChild(link);
            }
        });
    };
};
function loadAllAssets(opts) {
    return new Promise(function (resolve, reject) {
        var scriptsPromise = opts.scripts.reduce(function (prev, fileName) {
            return prev.then(loadScriptTag(opts.baseScriptUrl + "/" + fileName));
        }, Promise.resolve(undefined));
        var stylesPromise = opts.styles.reduce(function (prev, fileName) {
            return prev.then(loadLinkTag(opts.baseScriptUrl + "/" + fileName));
        }, Promise.resolve(undefined));
        Promise.all([scriptsPromise, stylesPromise]).then(resolve, reject);
    });
}
function loadAllAssetsForIframe(opts) {
    var iframeId = dom_utils_1.generateIFrameID(opts.name);
    var iframeEl = document.getElementById(iframeId);
    if (!iframeEl) {
        return new Promise(function (resolve, reject) {
            reject();
        });
    }
    return new Promise(function (resolve, reject) {
        var scriptsPromise = opts.scripts.reduce(function (prev, fileName) {
            return prev.then(loadScriptTag(opts.baseScriptUrl + "/" + fileName, iframeEl));
        }, Promise.resolve(undefined));
        var zonejsPromise = loadScriptPromise("/assets/zone.min.js", iframeEl);
        var stylesPromise = opts.styles.reduce(function (prev, fileName) {
            return prev.then(loadLinkTag(opts.baseScriptUrl + "/" + fileName));
        }, Promise.resolve(undefined));
        Promise.all([scriptsPromise, zonejsPromise, stylesPromise]).then(resolve, reject);
    });
}
var xmlToAssets = function (xml) {
    var dom = document.createElement('html');
    dom.innerHTML = xml;
    var linksEls = dom.querySelectorAll('link[rel="stylesheet"]');
    var scriptsEls = dom.querySelectorAll('script[type="text/javascript"]');
    return {
        styles: Array.from(linksEls).map(function (el) { return el.getAttribute('href'); }),
        scripts: Array.from(scriptsEls)
            .map(function (el) { return el.getAttribute('src'); })
            .filter(function (src) {
            if (src) {
                return !src.match(/zone\.js/);
            }
        })
    };
};
var transformOptsWithAssets = function (opts) {
    var url = opts.baseScriptUrl + "/index.html";
    return new Promise(function (resolve, reject) {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function (event) {
            if (req.readyState === XMLHttpRequest.DONE) {
                if (req.status >= 200 && req.status < 400) {
                    var res = xmlToAssets(req.responseText);
                    opts.styles = res.styles;
                    opts.scripts = res.scripts;
                    resolve(null);
                }
                else {
                    reject("Try to load " + url + ", status : " + req.status + " => " + req.statusText);
                }
            }
        };
        req.open('GET', url, true);
        req.send(null);
    });
};
var loadAllAssetsByUrl = function (opts) {
    return new Promise(function (resolve, reject) {
        transformOptsWithAssets(opts).then(function () {
            var scriptsPromise = opts.scripts.reduce(function (prev, fileName) {
                return prev.then(loadScriptTag(opts.baseScriptUrl + "/" + fileName));
            }, Promise.resolve(undefined));
            var stylesPromise = opts.styles.reduce(function (prev, fileName) {
                return prev.then(loadLinkTag(opts.baseScriptUrl + "/" + fileName));
            }, Promise.resolve(undefined));
            Promise.all([scriptsPromise, stylesPromise]).then(resolve, reject);
        }, reject);
    });
};
function unloadTag(opts, scriptName) {
    return function () {
        return new Promise(function (resolve, reject) {
            var tag = document.getElementById(app_helper_1.hashCode(opts.baseScriptUrl + "/" + scriptName));
            if (tag) {
                document.head.removeChild(tag);
            }
            resolve();
        });
    };
}
var MooaLoaderHelper = {
    loadAllAssets: loadAllAssets,
    loadAllAssetsByUrl: loadAllAssetsByUrl,
    loadAllAssetsForIframe: loadAllAssetsForIframe,
    unloadTag: unloadTag
};
exports.default = MooaLoaderHelper;
//# sourceMappingURL=mooa-loader.helper.js.map